@@ -26,8 +26,11 @@ def __exiting(signal, frame):
         port.exit()
     for thread in _threads:
         thread.exit()
-    sys.exit(0)
-    
+    try:
+        sys.exit(0)
+    except Exception as e:
+        print(e)
+
 signal.signal(signal.SIGINT, __exiting)
 piano = mbuild.piano
-sleep(1)
\ No newline at end of file
+sleep(0.5)
\ No newline at end of file
