@@ -39,7 +39,7 @@ class SerialPort():
             self._ser.timeout = 0.01
             sleep(2)
             self._thread = threading.Thread(target=self._on_read,args=(self._callback,))
-            # self._thread.daemon = True
+            self._thread.daemon = True
             self._thread.start()
             mkpiano.add_port(self)
         except Exception as e:
@@ -80,15 +80,23 @@ class SerialPort():
         if len(self._queue)>0:
             if self.is_open():
                 buf = self._queue[0]
-                self._ser.write(buf)
+                try:
+                    self._ser.write(buf)
+                except serial.serialutil.SerialException as e:
+                    self.exiting = True
+                    print("\033[1;33m连接失败，未检测到设备\033[0m")
+                    print('$#&*@{"err_code":102,"err_msg":"未检测到设备","device":"mkpiano","extra":{}}@*&#$')
+                    return None
                 self._queue.pop(0)
 
     def read(self):
-        return self._ser.read()
-        # try:
-        #     return self._ser.read()
-        # except serial.serialutil.SerialException:
-        #     return []
+        try:
+            return self._ser.read()
+        except serial.serialutil.SerialException as e:
+            self.exiting = True
+            print("\033[1;33m连接失败，未检测到设备\033[0m")
+            print('$#&*@{"err_code":102,"err_msg":"未检测到设备","device":"mkpiano","extra":{}}@*&#$')
+            return []
 
     def enable_sending(self):
         self._is_sending = True
@@ -108,6 +116,7 @@ class SerialPort():
         self._ser.close()
 
     def exit(self):
+        self._is_sending = False
         self.exiting = True
         self._thread.join()
         self.close()
