@@ -20,6 +20,7 @@ class _BaseModule:
     def setup(self,board,idx,mode=1,period=0):
         if board._autoconnect()==False:
             print("\033[1;33m连接失败，未检测到设备\033[0m")
+            print('$#&*@{"err_code":102,"err_msg":"未检测到设备","device":"mkpiano","extra":{}}@*&#$')
             self.quit()
         else:
             print("\033[1;32m连接成功，正在运行程序...\033[0m")
@@ -74,7 +75,7 @@ class _BaseModule:
         self._pack.data = pack.data
         self._is_received = True
 
-    def read(self,idx,service,subservice,data,callback):
+    def read(self,idx,service,subservice,data,callback=None):
         if not callback is None:
             self._callback = callback
         self._is_received = False
@@ -1196,6 +1197,7 @@ class FingertipPiano(_BaseModule):
         self._isExiting = False
         self._pack.service = 0x64
         self._pack.subservice = 0xf
+        self._hasCount = False
         self._piano = {'touch_strength':{},'key':{},'touch':{},'joystick':{},'distance':{},'key_count':{},'last_touched':{},'last_pressed':{},'gesture':{}}
 
     def __on_parse(self, pack):
@@ -1226,6 +1228,9 @@ class FingertipPiano(_BaseModule):
 
     def get_count_pressed(self,index=1,idx=None):
         subscribe_id = self.__subscribe(0x78,idx)
+        if self._hasCount == False:
+            self._hasCount = True
+            data = super().read(idx or self._pack.idx,self._pack.service,self._pack.subservice,[0x9])
         if subscribe_id in self._piano['key_count']:
             if index==1 or (type(index)==str and index.lower()=='a'):
                 return self._piano['key_count'][subscribe_id][0]
@@ -1246,10 +1251,16 @@ class FingertipPiano(_BaseModule):
     def is_obstacle_ahead(self):
         return self.distance<95
 
+    def get_obstacle_ahead(self):
+        return self.distance<95
+
     @property
     def obstacle_moving(self):
         return self.gesture
 
+    def get_obstacle_moving(self):
+        return self.gesture
+
     def last_pressed(self,idx=0):
         if idx==1:
             idx = 'A'
@@ -1291,6 +1302,9 @@ class FingertipPiano(_BaseModule):
                 return self._piano['touch'][subscribe_id][3]==1
         return False
 
+    def get_touched(self,index=1,idx=None):
+        return self.is_touched(index,idx)
+        
     def last_touched(self,idx=0):
         if idx==0:
             if self._piano['touch4']+self._piano['touch1']+self._piano['touch2']+self._piano['touch3']==0 and self._piano['last_touched']!=idx:
@@ -1360,19 +1374,15 @@ class FingertipPiano(_BaseModule):
             return self.DIRECTION_RIGHT
         return self.DIRECTION_HOME
 
-    @property
     def is_joystick_up(self):
         return self.get_joystick_y()>30
 
-    @property
     def is_joystick_down(self):
         return self.get_joystick_y()<-30
 
-    @property
     def is_joystick_left(self):
         return self.get_joystick_x()<-30
 
-    @property
     def is_joystick_right(self):
         return self.get_joystick_x()>30
     
@@ -1421,6 +1431,12 @@ class FingertipPiano(_BaseModule):
         return self.GESTURE_MOVING_UP
 
     def set_led(self,red=0,green=0,blue=0,idx=None):
+        if type(red)==str or red<0:
+            red = 0
+        if type(green)==str or green<0:
+            green = 0
+        if type(blue)==str or blue<0:
+            blue = 0
         pack = NeuronPackData()
         pack.idx = idx or self._pack.idx
         pack.service = self._pack.service
